import glob
import os
from pathlib import Path

import natsort
import numpy as np
import open3d as o3d
from scipy.spatial.transform import Rotation as R

from kiss_icp.config import KISSConfig, load_config
from kiss_icp.datasets.cache import get_cache, memoize


class ApolloDataset:
    def __init__(self, data_dir: Path, config: Path, *_, **__):
        super().__init__()
        self.config = load_config(config)
        self.scan_files = natsort.natsorted(glob.glob(f"{data_dir}/pcds/*.pcd"))
        self.gt_poses = self.readPoses(f"{data_dir}/poses/gt_poses.txt")
        self.sequence_id = os.path.basename(data_dir)

        # Cache
        self.use_cache = self.config.use_cache
        self.cache = get_cache(directory=os.path.join(self.__class__.__name__, self.sequence_id))

    def __len__(self):
        return len(self.scan_files)

    def __getitem__(self, idx):
        return self.get_scan(self.scan_files[idx], self.config.data)

    @memoize()
    def get_scan(self, scan_file: str, config: KISSConfig.data):
        points = np.asarray(o3d.io.read_point_cloud(scan_file).points, dtype=np.float64)
        points = self._preprocess(points, config) if config.preprocess else points
        return points.astype(np.float64)

    @staticmethod
    def _preprocess(points, config: KISSConfig.data):
        points = points[np.linalg.norm(points, axis=1) <= config.max_range]
        points = points[np.linalg.norm(points, axis=1) >= config.min_range]
        return points

    @staticmethod
    def readPoses(file):
        data = np.loadtxt(file)
        id, time, t, q = np.split(data, [1, 2, 5], axis=1)
        r = R.from_quat(q).as_matrix()
        pose = np.zeros([r.shape[0], 4, 4])
        pose[:, :3, -1] = t
        pose[:, :3, :3] = r
        pose[:, -1, -1] = 1
        return pose
