import glob
import os
from pathlib import Path

import natsort
import numpy as np
import open3d as o3d

from kiss_icp.config import KISSConfig, load_config
from kiss_icp.datasets.cache import get_cache, memoize


class GenericDataset:
    def __init__(self, data_dir: Path, config: Path, max_range: float, *_, **__):
        # Config stuff
        self.config = load_config(config)
        self.config.data.max_range = max_range if max_range else self.config.data.max_range
        self.sequence_id = os.path.basename(data_dir)
        self.scans_dir = os.path.join(os.path.realpath(data_dir), "")
        self.scan_files = np.array(natsort.natsorted(os.listdir(str(self.scans_dir))), dtype=np.str)

        # Cache
        self.use_cache = self.config.use_cache
        self.cache = get_cache(directory=os.path.join(self.__class__.__name__, self.sequence_id))

    def __len__(self):
        return len(self.scan_files)

    def __getitem__(self, idx):
        return self.read_point_cloud(
            os.path.join(self.scans_dir, self.scan_files[idx]), self.config.data
        )

    @memoize()
    def read_point_cloud(self, file_path: str, config: KISSConfig.data):
        if os.path.splitext(file_path)[-1] == ".bin":
            points = np.fromfile(file_path, dtype=np.float32).reshape((-1, 4))[:, :3]
        else:
            points = np.asarray(o3d.io.read_point_cloud(file_path).points, dtype=np.float64)
        points = self._preprocess(points, config) if config.preprocess else points
        return points.astype(np.float64)

    @staticmethod
    def _preprocess(points, config: KISSConfig.data):
        points = points[np.linalg.norm(points, axis=1) <= config.max_range]
        points = points[np.linalg.norm(points, axis=1) >= config.min_range]
        return points
