#!/usr/bin/env python3
from pathlib import Path

import typer

from kiss_icp.datasets import available_dataloaders


def name_callback(value: str):
    dl = available_dataloaders()
    if value not in dl:
        raise typer.BadParameter(f"Supported dataloaders are:\n{', '.join(dl)}")
    return value


def get_package_dir() -> str:
    import kiss_icp
    import os

    return os.path.dirname(kiss_icp.__file__)


def main(
    data: Path = typer.Argument(
        ...,
        show_default=False,
        help="The main directory for the dataloader",
    ),
    dataloader: str = typer.Option(
        "generic",
        show_default=False,
        case_sensitive=False,
        autocompletion=available_dataloaders,
        callback=name_callback,
        help="Use a specific dataloader from those supported by KISS-ICP",
    ),
    config: Path = typer.Option(
        f"{get_package_dir()}/config/default.yaml",
        "--config",
        "-c",
        exists=True,
        show_default=False,
        help="Path to the [Optional] configuration file",
    ),
    n_scans: int = typer.Option(
        -1,
        "--n-scans",
        "-n",
        show_default=False,
        help="Specify the number of scans to process, default is the entire dataset",
    ),
    jump: int = typer.Option(
        0,
        "--jump",
        "-j",
        show_default=False,
        help="Specify if you want to start to process scans from a given starting point",
    ),
    sequence: int = typer.Option(
        None,
        "--sequence",
        "-s",
        show_default=False,
        help="For some dataloaders, you need to specify a given sequence [kitti, kitti-raw]",
    ),
    max_range: float = typer.Option(
        None,
        "--max_range",
        "-r",
        show_default=False,
        help="Overrides the max_range from the default configuration",
    ),
    topic: str = typer.Option(
        None,
        "--topic",
        "-t",
        show_default=False,
        help="Only valid when processing rosbag files",
    ),
    deskew: bool = typer.Option(
        False,
        "--deskew",
        help="Wheter or not to deskew the scan using a constant-velocity model",
    ),
    visualize: bool = typer.Option(
        False,
        "--visualize",
        help="Open an online visualization of the KISS-ICP pipeline",
    ),
):
    """TOOD(NACHO): WILL WRITE A VITUFIL HELP HERE"""
    # Check if the main source is a bagfile, then automatically fallback to RosbagDataset
    dataloader = "rosbag" if data.is_file() and data.name.split(".")[-1] == "bag" else dataloader

    # Lazy-loading for faster CLI
    from kiss_icp.datasets import dataset_factory
    from kiss_icp.pipeline import OdometryPipeline

    OdometryPipeline(
        dataset=dataset_factory(
            dataloader=dataloader,
            data_dir=data,
            config=config,
            # Additional options
            max_range=max_range,
            sequence=sequence,
            topic=topic,
        ),
        config=config,
        deskew=deskew,
        visualize=visualize,
        n_scans=n_scans,
        jump=jump,
    ).run()


def run():
    typer.run(main)
